@@ -55,7 +55,6 @@
    When modifications are made, one should be careful that this behavior is
    preserved.
  */
-#include <config.h>
 #include <errno.h>
 #include <stdarg.h>
 #include <stdio.h>
@@ -84,6 +83,8 @@
 #define V4L2_MMAP_OFFSET_MAGIC      0xABCDEF00u
 
 static void v4l2_adjust_src_fmt_to_fps(int index, int fps);
+static void v4l2_set_src_and_dest_format(int index,
+		struct v4l2_format *src_fmt, struct v4l2_format *dest_fmt);
 
 static pthread_mutex_t v4l2_open_mutex = PTHREAD_MUTEX_INITIALIZER;
 static struct v4l2_dev_info devices[V4L2_MAX_DEVICES] = {
@@ -94,6 +95,32 @@ static struct v4l2_dev_info devices[V4L2_MAX_DEVICES] = {
 };
 static int devices_used;
 
+static int v4l2_ensure_convert_mmap_buf(int index)
+{
+	if (devices[index].convert_mmap_buf != MAP_FAILED) {
+		return 0;
+	}
+
+	devices[index].convert_mmap_buf_size =
+		devices[index].convert_mmap_frame_size * devices[index].no_frames;
+
+	devices[index].convert_mmap_buf = (void *)SYS_MMAP(NULL,
+			devices[index].convert_mmap_buf_size,
+			PROT_READ | PROT_WRITE,
+			MAP_ANONYMOUS | MAP_PRIVATE,
+			-1, 0);
+
+	if (devices[index].convert_mmap_buf == MAP_FAILED) {
+		devices[index].convert_mmap_buf_size = 0;
+
+		int saved_err = errno;
+		V4L2_LOG_ERR("allocating conversion buffer\n");
+		errno = saved_err;
+		return -1;
+	}
+
+	return 0;
+}
 
 static int v4l2_request_read_buffers(int index)
 {
@@ -158,13 +185,14 @@ static int v4l2_map_buffers(int index)
 		buf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
 		buf.memory = V4L2_MEMORY_MMAP;
 		buf.index = i;
+		buf.reserved = buf.reserved2 = 0;
 		result = devices[index].dev_ops->ioctl(
 				devices[index].dev_ops_priv,
 				devices[index].fd, VIDIOC_QUERYBUF, &buf);
 		if (result) {
 			int saved_err = errno;
 
-			V4L2_LOG_ERR("querying buffer %u: %s\n", i, strerror(errno));
+			V4L2_PERROR("querying buffer %u", i);
 			errno = saved_err;
 			break;
 		}
@@ -175,7 +203,7 @@ static int v4l2_map_buffers(int index)
 		if (devices[index].frame_pointers[i] == MAP_FAILED) {
 			int saved_err = errno;
 
-			V4L2_LOG_ERR("mmapping buffer %u: %s\n", i, strerror(errno));
+			V4L2_PERROR("mmapping buffer %u", i);
 			errno = saved_err;
 			result = -1;
 			break;
@@ -216,7 +244,7 @@ static int v4l2_streamon(int index)
 		if (result) {
 			int saved_err = errno;
 
-			V4L2_LOG_ERR("turning on stream: %s\n", strerror(errno));
+			V4L2_PERROR("turning on stream");
 			errno = saved_err;
 			return result;
 		}
@@ -239,7 +267,7 @@ static int v4l2_streamoff(int index)
 		if (result) {
 			int saved_err = errno;
 
-			V4L2_LOG_ERR("turning off stream: %s\n", strerror(errno));
+			V4L2_PERROR("turning off stream");
 			errno = saved_err;
 			return result;
 		}
@@ -269,7 +297,7 @@ static int v4l2_queue_read_buffer(int index, int buffer_index)
 	if (result) {
 		int saved_err = errno;
 
-		V4L2_LOG_ERR("queuing buf %d: %s\n", buffer_index, strerror(errno));
+		V4L2_PERROR("queuing buf %d", buffer_index);
 		errno = saved_err;
 		return result;
 	}
@@ -282,7 +310,7 @@ static int v4l2_dequeue_and_convert(int index, struct v4l2_buffer *buf,
 		unsigned char *dest, int dest_size)
 {
 	const int max_tries = V4L2_IGNORE_FIRST_FRAME_ERRORS + 1;
-	int result, tries = max_tries;
+	int result, tries = max_tries, frame_info_gen;
 
 	/* Make sure we have the real v4l2 buffers mapped */
 	result = v4l2_map_buffers(index);
@@ -290,14 +318,17 @@ static int v4l2_dequeue_and_convert(int index, struct v4l2_buffer *buf,
 		return result;
 
 	do {
+		frame_info_gen = devices[index].frame_info_generation;
+		pthread_mutex_unlock(&devices[index].stream_lock);
 		result = devices[index].dev_ops->ioctl(
 				devices[index].dev_ops_priv,
 				devices[index].fd, VIDIOC_DQBUF, buf);
+		pthread_mutex_lock(&devices[index].stream_lock);
 		if (result) {
 			if (errno != EAGAIN) {
 				int saved_err = errno;
 
-				V4L2_LOG_ERR("dequeuing buf: %s\n", strerror(errno));
+				V4L2_PERROR("dequeuing buf");
 				errno = saved_err;
 			}
 			return result;
@@ -305,11 +336,17 @@ static int v4l2_dequeue_and_convert(int index, struct v4l2_buffer *buf,
 
 		devices[index].frame_queued &= ~(1 << buf->index);
 
+		if (frame_info_gen != devices[index].frame_info_generation) {
+			errno = -EINVAL;
+			return -1;
+		}
+
 		result = v4lconvert_convert(devices[index].convert,
 				&devices[index].src_fmt, &devices[index].dest_fmt,
 				devices[index].frame_pointers[buf->index],
 				buf->bytesused, dest ? dest : (devices[index].convert_mmap_buf +
-					buf->index * V4L2_FRAME_BUF_SIZE), dest_size);
+					buf->index * devices[index].convert_mmap_frame_size),
+				dest_size);
 
 		if (devices[index].first_frame) {
 			/* Always treat convert errors as EAGAIN during the first few frames, as
@@ -385,7 +422,7 @@ static int v4l2_read_and_convert(int index, unsigned char *dest, int dest_size)
 			if (result && errno != EAGAIN) {
 				int saved_err = errno;
 
-				V4L2_LOG_ERR("reading: %s\n", strerror(errno));
+				V4L2_PERROR("reading");
 				errno = saved_err;
 			}
 			return result;
@@ -518,7 +555,7 @@ static void v4l2_set_conversion_buf_params(int index, struct v4l2_buffer *buf)
 		buf->index = 0;
 
 	buf->m.offset = V4L2_MMAP_OFFSET_MAGIC | buf->index;
-	buf->length = V4L2_FRAME_BUF_SIZE;
+	buf->length = devices[index].convert_mmap_frame_size;
 	if (devices[index].frame_map_count[buf->index])
 		buf->flags |= V4L2_BUF_FLAG_MAPPED;
 	else
@@ -537,13 +574,14 @@ static int v4l2_buffers_mapped(int index)
 			buf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
 			buf.memory = V4L2_MEMORY_MMAP;
 			buf.index = i;
+			buf.reserved = buf.reserved2 = 0;
 			if (devices[index].dev_ops->ioctl(
 					devices[index].dev_ops_priv,
 					devices[index].fd, VIDIOC_QUERYBUF,
 					&buf)) {
 				int saved_err = errno;
 
-				V4L2_LOG_ERR("querying buffer %u: %s\n", i, strerror(errno));
+				V4L2_PERROR("querying buffer %u", i);
 				errno = saved_err;
 				break;
 			}
@@ -620,6 +658,7 @@ int v4l2_fd_open(int fd, int v4l2_flags)
 	void *plugin_library;
 	void *dev_ops_priv;
 	const struct libv4l_dev_ops *dev_ops;
+	long page_size;
 
 	v4l2_plugin_init(fd, &plugin_library, &dev_ops_priv, &dev_ops);
 
@@ -631,6 +670,17 @@ int v4l2_fd_open(int fd, int v4l2_flags)
 			v4l2_log_file = fopen(lfname, "w");
 	}
 
+	/* Get page_size (for mmap emulation) */
+	page_size = sysconf(_SC_PAGESIZE);
+	if (page_size < 0) {
+		int saved_err = errno;
+		V4L2_LOG_ERR("unable to retrieve page size: %s\n",
+			     strerror(errno));
+		v4l2_plugin_cleanup(plugin_library, dev_ops_priv, dev_ops);
+		errno = saved_err;
+		return -1;
+	}
+
 	/* check that this is a v4l2 device */
 	if (dev_ops->ioctl(dev_ops_priv, fd, VIDIOC_QUERYCAP, &cap)) {
 		int saved_err = errno;
@@ -688,7 +738,7 @@ no_capture:
 	pthread_mutex_unlock(&v4l2_open_mutex);
 
 	if (index == V4L2_MAX_DEVICES) {
-		V4L2_LOG_ERR("attempting to open more then %d video devices\n",
+		V4L2_LOG_ERR("attempting to open more than %d video devices\n",
 				V4L2_MAX_DEVICES);
 		v4l2_plugin_cleanup(plugin_library, dev_ops_priv, dev_ops);
 		errno = EBUSY;
@@ -708,18 +758,11 @@ no_capture:
 	    (parm.parm.capture.capability & V4L2_CAP_TIMEPERFRAME))
 		devices[index].flags |= V4L2_SUPPORTS_TIMEPERFRAME;
 	devices[index].open_count = 1;
-	devices[index].src_fmt = fmt;
+	devices[index].page_size = page_size;
+	devices[index].src_fmt  = fmt;
 	devices[index].dest_fmt = fmt;
-
-	/* When a user does a try_fmt with the current dest_fmt and the dest_fmt
-	   is a supported one we will align the resolution (see try_fmt for why).
-	   Do the same here now, so that a try_fmt on the result of a get_fmt done
-	   immediately after open leaves the fmt unchanged. */
-	if (v4lconvert_supported_dst_format(
-				devices[index].dest_fmt.fmt.pix.pixelformat)) {
-		devices[index].dest_fmt.fmt.pix.width &= ~7;
-		devices[index].dest_fmt.fmt.pix.height &= ~1;
-	}
+	v4l2_set_src_and_dest_format(index, &devices[index].src_fmt,
+				     &devices[index].dest_fmt);
 
 	pthread_mutex_init(&devices[index].stream_lock, NULL);
 
@@ -727,6 +770,7 @@ no_capture:
 	devices[index].nreadbuffers = V4L2_DEFAULT_NREADBUFFERS;
 	devices[index].convert = convert;
 	devices[index].convert_mmap_buf = MAP_FAILED;
+	devices[index].convert_mmap_buf_size = 0;
 	for (i = 0; i < V4L2_MAX_NO_FRAMES; i++) {
 		devices[index].frame_pointers[i] = MAP_FAILED;
 		devices[index].frame_map_count[i] = 0;
@@ -740,7 +784,7 @@ no_capture:
 
 	/* Note we always tell v4lconvert to optimize src fmt selection for
 	   our default fps, the only exception is the app explicitly selecting
-	   a fram erate using the S_PARM ioctl after a S_FMT */
+	   a frame rate using the S_PARM ioctl after a S_FMT */
 	if (devices[index].convert)
 		v4lconvert_set_fps(devices[index].convert, V4L2_DEFAULT_FPS);
 	v4l2_update_fps(index, &parm);
@@ -795,12 +839,15 @@ int v4l2_close(int fd)
 	/* Free resources */
 	v4l2_unmap_buffers(index);
 	if (devices[index].convert_mmap_buf != MAP_FAILED) {
-		if (v4l2_buffers_mapped(index))
-			V4L2_LOG_WARN("v4l2 mmap buffers still mapped on close()\n");
-		else
+		if (v4l2_buffers_mapped(index)) {
+			if (!devices[index].gone)
+				V4L2_LOG_WARN("v4l2 mmap buffers still mapped on close()\n");
+		} else {
 			SYS_MUNMAP(devices[index].convert_mmap_buf,
-					devices[index].no_frames * V4L2_FRAME_BUF_SIZE);
+					devices[index].convert_mmap_buf_size);
+		}
 		devices[index].convert_mmap_buf = MAP_FAILED;
+		devices[index].convert_mmap_buf_size = 0;
 	}
 	v4lconvert_destroy(devices[index].convert);
 	free(devices[index].readbuf);
@@ -837,6 +884,7 @@ int v4l2_dup(int fd)
 
 static int v4l2_check_buffer_change_ok(int index)
 {
+	devices[index].frame_info_generation++;
 	v4l2_unmap_buffers(index);
 
 	/* Check if the app itself still is using the stream */
@@ -853,8 +901,9 @@ static int v4l2_check_buffer_change_ok(int index)
 	   v4l2_unrequest_read_buffers may change the no_frames, so free the
 	   convert mmap buffer */
 	SYS_MUNMAP(devices[index].convert_mmap_buf,
-			devices[index].no_frames * V4L2_FRAME_BUF_SIZE);
+			devices[index].convert_mmap_buf_size);
 	devices[index].convert_mmap_buf = MAP_FAILED;
+	devices[index].convert_mmap_buf_size = 0;
 
 	if (devices[index].flags & V4L2_STREAM_CONTROLLED_BY_READ) {
 		V4L2_LOG("deactivating read-stream for settings change\n");
@@ -888,20 +937,40 @@ static int v4l2_pix_fmt_identical(struct v4l2_format *a, struct v4l2_format *b)
 static void v4l2_set_src_and_dest_format(int index,
 		struct v4l2_format *src_fmt, struct v4l2_format *dest_fmt)
 {
+	/*
+	 * When a user does a try_fmt with the current dest_fmt and the
+	 * dest_fmt is a supported one we will align the resolution (see
+	 * libv4lconvert_try_fmt). We do this here too, in case dest_fmt gets
+	 * set without having gone through libv4lconvert_try_fmt, so that a
+	 * try_fmt on the result of a get_fmt always returns the same result.
+	 */
+	if (v4lconvert_supported_dst_format(dest_fmt->fmt.pix.pixelformat)) {
+		dest_fmt->fmt.pix.width &= ~7;
+		dest_fmt->fmt.pix.height &= ~1;
+	}
+
 	/* Sigh some drivers (pwc) do not properly reflect what one really gets
 	   after a s_fmt in their try_fmt answer. So update dest format (which we
 	   report as result from s_fmt / g_fmt to the app) with all info from the src
 	   format not changed by conversion */
 	dest_fmt->fmt.pix.field = src_fmt->fmt.pix.field;
 	dest_fmt->fmt.pix.colorspace = src_fmt->fmt.pix.colorspace;
+	dest_fmt->fmt.pix.xfer_func = src_fmt->fmt.pix.xfer_func;
+	dest_fmt->fmt.pix.ycbcr_enc = src_fmt->fmt.pix.ycbcr_enc;
+	dest_fmt->fmt.pix.quantization = src_fmt->fmt.pix.quantization;
 	/* When we're not converting use bytesperline and imagesize from src_fmt */
 	if (v4l2_pix_fmt_compat(src_fmt, dest_fmt)) {
 		dest_fmt->fmt.pix.bytesperline = src_fmt->fmt.pix.bytesperline;
 		dest_fmt->fmt.pix.sizeimage = src_fmt->fmt.pix.sizeimage;
-	}
+	} else
+		v4lconvert_fixup_fmt(dest_fmt);
 
 	devices[index].src_fmt = *src_fmt;
 	devices[index].dest_fmt = *dest_fmt;
+	/* round up to full page size */
+	devices[index].convert_mmap_frame_size =
+		(((dest_fmt->fmt.pix.sizeimage + devices[index].page_size - 1)
+		/ devices[index].page_size) * devices[index].page_size);
 }
 
 static int v4l2_s_fmt(int index, struct v4l2_format *dest_fmt)
@@ -949,7 +1018,7 @@ static int v4l2_s_fmt(int index, struct v4l2_format *dest_fmt)
 					       VIDIOC_S_FMT, &src_fmt);
 	if (result) {
 		int saved_err = errno;
-		V4L2_LOG_ERR("setting pixformat: %s\n", strerror(errno));
+		V4L2_PERROR("setting pixformat");
 		/* Report to the app dest_fmt has not changed */
 		*dest_fmt = devices[index].dest_fmt;
 		errno = saved_err;
@@ -961,7 +1030,7 @@ static int v4l2_s_fmt(int index, struct v4l2_format *dest_fmt)
 	if (src_fmt.fmt.pix.width != req_pix_fmt.width ||
 	    src_fmt.fmt.pix.height != req_pix_fmt.height ||
 	    src_fmt.fmt.pix.pixelformat != req_pix_fmt.pixelformat) {
-		V4L2_LOG_ERR("set_fmt gave us a different result then try_fmt!\n");
+		V4L2_LOG_ERR("set_fmt gave us a different result than try_fmt!\n");
 		/* Not what we expected / wanted, disable conversion */
 		*dest_fmt = src_fmt;
 	}
@@ -1011,6 +1080,9 @@ int v4l2_ioctl(int fd, unsigned long int request, ...)
 	case VIDIOC_QUERYCTRL:
 	case VIDIOC_G_CTRL:
 	case VIDIOC_S_CTRL:
+	case VIDIOC_G_EXT_CTRLS:
+	case VIDIOC_TRY_EXT_CTRLS:
+	case VIDIOC_S_EXT_CTRLS:
 	case VIDIOC_ENUM_FRAMESIZES:
 	case VIDIOC_ENUM_FRAMEINTERVALS:
 		is_capture_request = 1;
@@ -1118,6 +1190,18 @@ no_capture_request:
 		result = v4lconvert_vidioc_s_ctrl(devices[index].convert, arg);
 		break;
 
+	case VIDIOC_G_EXT_CTRLS:
+		result = v4lconvert_vidioc_g_ext_ctrls(devices[index].convert, arg);
+		break;
+
+	case VIDIOC_TRY_EXT_CTRLS:
+		result = v4lconvert_vidioc_try_ext_ctrls(devices[index].convert, arg);
+		break;
+
+	case VIDIOC_S_EXT_CTRLS:
+		result = v4lconvert_vidioc_s_ext_ctrls(devices[index].convert, arg);
+		break;
+
 	case VIDIOC_QUERYCAP: {
 		struct v4l2_capability *cap = arg;
 
@@ -1167,7 +1251,9 @@ no_capture_request:
 	case VIDIOC_S_STD:
 	case VIDIOC_S_INPUT:
 	case VIDIOC_S_DV_TIMINGS: {
-		struct v4l2_format src_fmt;
+		struct v4l2_format src_fmt = { 0 };
+		unsigned int orig_dest_pixelformat =
+			devices[index].dest_fmt.fmt.pix.pixelformat;
 
 		result = devices[index].dev_ops->ioctl(
 				devices[index].dev_ops_priv,
@@ -1181,9 +1267,8 @@ no_capture_request:
 				devices[index].dev_ops_priv,
 				fd, VIDIOC_G_FMT, &src_fmt);
 		if (result) {
-			V4L2_LOG_ERR("getting pixformat after %s: %s\n",
-				     v4l2_ioctls[_IOC_NR(request)],
-				     strerror(errno));
+			V4L2_PERROR("getting pixformat after %s",
+				     v4l2_ioctls[_IOC_NR(request)]);
 			result = 0; /* The original command did succeed */
 			break;
 		}
@@ -1197,9 +1282,10 @@ no_capture_request:
 		/* The fmt has been changed, remember the new format ... */
 		devices[index].src_fmt  = src_fmt;
 		devices[index].dest_fmt = src_fmt;
+		v4l2_set_src_and_dest_format(index, &devices[index].src_fmt,
+					     &devices[index].dest_fmt);
 		/* and try to restore the last set destination pixelformat. */
-		src_fmt.fmt.pix.pixelformat =
-			devices[index].dest_fmt.fmt.pix.pixelformat;
+		src_fmt.fmt.pix.pixelformat = orig_dest_pixelformat;
 		result = v4l2_s_fmt(index, &src_fmt);
 		if (result) {
 			V4L2_LOG_WARN("restoring destination pixelformat after %s failed\n",
@@ -1224,7 +1310,7 @@ no_capture_request:
 		if (result)
 			break;
 
-		/* No more buffers then we can manage please */
+		/* No more buffers than we can manage please */
 		if (req->count > V4L2_MAX_NO_FRAMES)
 			req->count = V4L2_MAX_NO_FRAMES;
 
@@ -1293,12 +1379,14 @@ no_capture_request:
 		}
 
 		if (!v4l2_needs_conversion(index)) {
+			pthread_mutex_unlock(&devices[index].stream_lock);
 			result = devices[index].dev_ops->ioctl(
 					devices[index].dev_ops_priv,
 					fd, VIDIOC_DQBUF, buf);
+			pthread_mutex_lock(&devices[index].stream_lock);
 			if (result) {
 				saved_err = errno;
-				V4L2_LOG_ERR("dequeuing buf: %s\n", strerror(errno));
+				V4L2_PERROR("dequeuing buf");
 				errno = saved_err;
 			}
 			break;
@@ -1307,23 +1395,12 @@ no_capture_request:
 		/* An application can do a DQBUF before mmap-ing in the buffer,
 		   but we need the buffer _now_ to write our converted data
 		   to it! */
-		if (devices[index].convert_mmap_buf == MAP_FAILED) {
-			devices[index].convert_mmap_buf = (void *)SYS_MMAP(NULL,
-				(size_t)(devices[index].no_frames * V4L2_FRAME_BUF_SIZE),
-				PROT_READ | PROT_WRITE,
-				MAP_ANONYMOUS | MAP_PRIVATE,
-				-1, 0);
-			if (devices[index].convert_mmap_buf == MAP_FAILED) {
-				saved_err = errno;
-				V4L2_LOG_ERR("allocating conversion buffer\n");
-				errno = saved_err;
-				result = -1;
-				break;
-			}
-		}
+		result = v4l2_ensure_convert_mmap_buf(index);
+		if (result)
+			break;
 
 		result = v4l2_dequeue_and_convert(index, buf, 0,
-						  V4L2_FRAME_BUF_SIZE);
+				devices[index].convert_mmap_frame_size);
 		if (result >= 0) {
 			buf->bytesused = result;
 			result = 0;
@@ -1443,7 +1520,7 @@ static void v4l2_adjust_src_fmt_to_fps(int index, int fps)
 	req_pix_fmt = src_fmt.fmt.pix;
 	if (devices[index].dev_ops->ioctl(devices[index].dev_ops_priv,
 			devices[index].fd, VIDIOC_S_FMT, &src_fmt)) {
-		V4L2_LOG_ERR("restoring src fmt: %s\n", strerror(errno));
+		V4L2_PERROR("restoring src fmt");
 		return;
 	}
 	v4l2_set_src_and_dest_format(index, &src_fmt, &dest_fmt);
@@ -1548,7 +1625,7 @@ void *v4l2_mmap(void *start, size_t length, int prot, int flags, int fd,
 	if (index == -1 ||
 			/* Check if the mmap data matches our answer to QUERY_BUF. If it doesn't,
 			   let the kernel handle it (to allow for mmap-based non capture use) */
-			start || length != V4L2_FRAME_BUF_SIZE ||
+			start || length != devices[index].convert_mmap_frame_size ||
 			((unsigned int)offset & ~0xFFu) != V4L2_MMAP_OFFSET_MAGIC) {
 		if (index != -1)
 			V4L2_LOG("Passing mmap(%p, %d, ..., %x, through to the driver\n",
@@ -1573,26 +1650,16 @@ void *v4l2_mmap(void *start, size_t length, int prot, int flags, int fd,
 		goto leave;
 	}
 
-	if (devices[index].convert_mmap_buf == MAP_FAILED) {
-		devices[index].convert_mmap_buf = (void *)SYS_MMAP(NULL,
-			(size_t)(devices[index].no_frames * V4L2_FRAME_BUF_SIZE),
-			PROT_READ | PROT_WRITE,
-			MAP_ANONYMOUS | MAP_PRIVATE,
-			-1, 0);
-		if (devices[index].convert_mmap_buf == MAP_FAILED) {
-			int saved_err = errno;
-
-			V4L2_LOG_ERR("allocating conversion buffer\n");
-			errno = saved_err;
-			result = MAP_FAILED;
-			goto leave;
-		}
+	if (v4l2_ensure_convert_mmap_buf(index)) {
+		errno = EINVAL;
+		result = MAP_FAILED;
+		goto leave;
 	}
 
 	devices[index].frame_map_count[buffer_index]++;
 
 	result = devices[index].convert_mmap_buf +
-		buffer_index * V4L2_FRAME_BUF_SIZE;
+		buffer_index * devices[index].convert_mmap_frame_size;
 
 	V4L2_LOG("Fake (conversion) mmap buf %u, seen by app at: %p\n",
 			buffer_index, result);
@@ -1610,10 +1677,11 @@ int v4l2_munmap(void *_start, size_t length)
 	unsigned char *start = _start;
 
 	/* Is this memory ours? */
-	if (start != MAP_FAILED && length == V4L2_FRAME_BUF_SIZE) {
+	if (start != MAP_FAILED) {
 		for (index = 0; index < devices_used; index++)
 			if (devices[index].fd != -1 &&
 					devices[index].convert_mmap_buf != MAP_FAILED &&
+					length == devices[index].convert_mmap_frame_size &&
 					start >= devices[index].convert_mmap_buf &&
 					(start - devices[index].convert_mmap_buf) % length == 0)
 				break;
@@ -1627,6 +1695,7 @@ int v4l2_munmap(void *_start, size_t length)
 
 			/* Re-do our checks now that we have the lock, things may have changed */
 			if (devices[index].convert_mmap_buf != MAP_FAILED &&
+					length == devices[index].convert_mmap_frame_size &&
 					start >= devices[index].convert_mmap_buf &&
 					(start - devices[index].convert_mmap_buf) % length == 0 &&
 					buffer_index < devices[index].no_frames) {
@@ -1672,7 +1741,7 @@ int v4l2_set_control(int fd, int cid, int value)
 		if (qctrl.type == V4L2_CTRL_TYPE_BOOLEAN)
 			ctrl.value = value ? 1 : 0;
 		else
-			ctrl.value = (value * (qctrl.maximum - qctrl.minimum) + 32767) / 65535 +
+			ctrl.value = ((long long) value * (qctrl.maximum - qctrl.minimum) + 32767) / 65535 +
 				qctrl.minimum;
 
 		result = v4lconvert_vidioc_s_ctrl(devices[index].convert, &ctrl);
@@ -1704,7 +1773,7 @@ int v4l2_get_control(int fd, int cid)
 	if (v4lconvert_vidioc_g_ctrl(devices[index].convert, &ctrl))
 		return -1;
 
-	return ((ctrl.value - qctrl.minimum) * 65535 +
+	return (((long long) ctrl.value - qctrl.minimum) * 65535 +
 			(qctrl.maximum - qctrl.minimum) / 2) /
 		(qctrl.maximum - qctrl.minimum);
 }
