@@ -25,7 +25,7 @@
  *
  * Note that the original bayer.c in libdc1394 supports many different
  * bayer decode algorithms, for lib4lconvert the one in this file has been
- * chosen (and optimized a bit) and the other algorithm's have been removed,
+ * chosen (and optimized a bit) and the other algorithms have been removed,
  * see bayer.c from libdc1394 for all supported algorithms
  */
 
@@ -631,3 +631,43 @@ void v4lconvert_bayer_to_yuv420(const unsigned char *bayer, unsigned char *yuv,
 	v4lconvert_border_bayer_line_to_y(bayer + stride, bayer, ydst, width,
 			!start_with_green, !blue_line);
 }
+
+void v4lconvert_bayer10_to_bayer8(void *bayer10,
+		unsigned char *bayer8, int width, int height)
+{
+	int i;
+	uint16_t *src = bayer10;
+
+	for (i = 0; i < width * height; i++)
+		bayer8[i] = src[i] >> 2;
+}
+
+void v4lconvert_bayer10p_to_bayer8(unsigned char *bayer10p,
+		unsigned char *bayer8, int width, int height)
+{
+	unsigned long i;
+	unsigned long len = width * height;
+
+	for (i = 0; i < len ; i += 4) {
+		/*
+		 * Do not use a second loop, hoping that
+		 * a clever compiler with understand the
+		 * pattern and will optimize it.
+		 */
+		bayer8[0] = bayer10p[0];
+		bayer8[1] = bayer10p[1];
+		bayer8[2] = bayer10p[2];
+		bayer8[3] = bayer10p[3];
+		bayer10p += 5;
+		bayer8 += 4;
+	}
+}
+
+void v4lconvert_bayer16_to_bayer8(unsigned char *bayer16,
+		unsigned char *bayer8, int width, int height)
+{
+	int i;
+
+	for (i = 0; i < width * height; i++)
+		bayer8[i] = bayer16[2*i+1];
+}
