@@ -51,7 +51,7 @@ static QString pixfmt2s(unsigned id)
 	pixfmt += (char)((id >> 8) & 0x7f);
 	pixfmt += (char)((id >> 16) & 0x7f);
 	pixfmt += (char)((id >> 24) & 0x7f);
-	if (id & (1 << 31))
+	if (id & (1U << 31))
 		pixfmt += "-BE";
 	return pixfmt;
 }
@@ -430,8 +430,11 @@ void GeneralTab::inputSection(v4l2_input vin)
 		QLabel *l = new QLabel("Refresh Tuner Status", parentWidget());
 		QWidget *w = new QWidget(parentWidget());
 		QHBoxLayout *box = new QHBoxLayout(w);
-
+#if QT_VERSION < 0x060000
 		box->setMargin(0);
+#else
+		box->setContentsMargins(0, 0, 0, 0);
+#endif
 		m_detectSubchans = new QToolButton(w);
 		m_detectSubchans->setIcon(QIcon(":/enterbutt.png"));
 		m_subchannels = new QLabel("", w);
@@ -746,7 +749,7 @@ void GeneralTab::formatSection(v4l2_fmtdesc fmt)
 		m_colorspace->addItem("SMPTE 170M", QVariant(V4L2_COLORSPACE_SMPTE170M));
 		m_colorspace->addItem("Rec. 709", QVariant(V4L2_COLORSPACE_REC709));
 		m_colorspace->addItem("sRGB", QVariant(V4L2_COLORSPACE_SRGB));
-		m_colorspace->addItem("Adobe RGB", QVariant(V4L2_COLORSPACE_ADOBERGB));
+		m_colorspace->addItem("opRGB", QVariant(V4L2_COLORSPACE_OPRGB));
 		m_colorspace->addItem("BT.2020", QVariant(V4L2_COLORSPACE_BT2020));
 		m_colorspace->addItem("DCI-P3", QVariant(V4L2_COLORSPACE_DCI_P3));
 		m_colorspace->addItem("SMPTE 240M", QVariant(V4L2_COLORSPACE_SMPTE240M));
@@ -761,7 +764,7 @@ void GeneralTab::formatSection(v4l2_fmtdesc fmt)
 		m_xferFunc->addItem(m_isOutput ? "Default" : "Autodetect", QVariant(V4L2_XFER_FUNC_DEFAULT));
 		m_xferFunc->addItem("Rec. 709", QVariant(V4L2_XFER_FUNC_709));
 		m_xferFunc->addItem("sRGB", QVariant(V4L2_XFER_FUNC_SRGB));
-		m_xferFunc->addItem("Adobe RGB", QVariant(V4L2_XFER_FUNC_ADOBERGB));
+		m_xferFunc->addItem("opRGB", QVariant(V4L2_XFER_FUNC_OPRGB));
 		m_xferFunc->addItem("DCI-P3", QVariant(V4L2_XFER_FUNC_DCI_P3));
 		m_xferFunc->addItem("SMPTE 2084", QVariant(V4L2_XFER_FUNC_SMPTE2084));
 		m_xferFunc->addItem("SMPTE 240M", QVariant(V4L2_XFER_FUNC_SMPTE240M));
@@ -777,12 +780,13 @@ void GeneralTab::formatSection(v4l2_fmtdesc fmt)
 		m_ycbcrEnc->addItem("Rec. 709", QVariant(V4L2_YCBCR_ENC_709));
 		m_ycbcrEnc->addItem("xvYCC 601", QVariant(V4L2_YCBCR_ENC_XV601));
 		m_ycbcrEnc->addItem("xvYCC 709", QVariant(V4L2_YCBCR_ENC_XV709));
-		m_ycbcrEnc->addItem("sYCC", QVariant(V4L2_YCBCR_ENC_SYCC));
 		m_ycbcrEnc->addItem("BT.2020", QVariant(V4L2_YCBCR_ENC_BT2020));
 		m_ycbcrEnc->addItem("BT.2020 Constant Luminance", QVariant(V4L2_YCBCR_ENC_BT2020_CONST_LUM));
 		m_ycbcrEnc->addItem("SMPTE 240M", QVariant(V4L2_YCBCR_ENC_SMPTE240M));
+		m_ycbcrEnc->addItem("HSV with Hue 0-179", QVariant(V4L2_HSV_ENC_180));
+		m_ycbcrEnc->addItem("HSV with Hue 0-255", QVariant(V4L2_HSV_ENC_256));
 
-		addLabel("Y'CbCr Encoding");
+		addLabel("Y'CbCr/HSV Encoding");
 		addWidget(m_ycbcrEnc);
 		connect(m_ycbcrEnc, SIGNAL(activated(int)), SLOT(ycbcrEncChanged(int)));
 
@@ -1083,7 +1087,10 @@ bool GeneralTab::createAudioDeviceList()
 
 		void **hint;
 
-		snd_device_name_hint(deviceNum, "pcm", &hint);
+		if (snd_device_name_hint(deviceNum, "pcm", &hint)) {
+			audioDevices--;
+			continue;
+		}
 		for (int i = 0; hint[i] != NULL; i++) {
 			int addAs = addAudioDevice(hint[i], deviceNum);
 			if (addAs == AUDIO_ADD_READ || addAs == AUDIO_ADD_READWRITE)
@@ -1103,7 +1110,7 @@ bool GeneralTab::createAudioDeviceList()
 
 void GeneralTab::changeAudioDevice()
 {
-	m_audioOutDevice->setEnabled(getAudioInDevice() != NULL ? getAudioInDevice().compare("None") : false);
+	m_audioOutDevice->setEnabled(getAudioInDevice() != nullptr ? getAudioInDevice().compare("None") : false);
 	emit audioDeviceChanged();
 }
 
@@ -1802,7 +1809,12 @@ void GeneralTab::updateStandard()
 	if ((vs.id & std) == 0)
 		return;
 	m_tvStandard->setCurrentIndex(vs.index);
-	what.sprintf("TV Standard (0x%llX)\n"
+#if QT_VERSION < 0x050500
+	what.sprintf(
+#else
+	what.asprintf(
+#endif
+		"TV Standard (0x%llX)\n"
 		"Frame period: %f (%d/%d)\n"
 		"Frame lines: %d", (long long int)std,
 		(double)vs.frameperiod.numerator / vs.frameperiod.denominator,
@@ -1872,7 +1884,12 @@ void GeneralTab::updateTimings()
 	if (memcmp(&timings, &p.timings, sizeof(timings)))
 		return;
 	m_videoTimings->setCurrentIndex(p.index);
-	what.sprintf("Video Timings (%u)\n"
+#if QT_VERSION < 0x050500
+	what.sprintf(
+#else
+	what.asprintf(
+#endif
+		"Video Timings (%u)\n"
 		"Frame %ux%u\n",
 		p.index, p.timings.bt.width, p.timings.bt.height);
 	m_isSDTV = p.timings.bt.width <= 720 && p.timings.bt.height <= 576;
@@ -2185,6 +2202,11 @@ void GeneralTab::updateFrameSize()
 	m_discreteSizes = false;
 	if (m_frameSize)
 		m_frameSize->setEnabled(false);
+	if (!m_frameWidth) {
+		updateFrameInterval();
+		return;
+	}
+
 	m_frameWidth->setEnabled(!m_haveBuffers);
 	m_frameWidth->blockSignals(true);
 	m_frameWidth->setMinimum(frmsize.stepwise.min_width);
@@ -2371,8 +2393,14 @@ int GeneralTab::checkMatchAudioDevice(void *md, const char *vid, enum device_typ
 
 	while ((devname = get_associated_device(md, devname, type, vid, dtype)) != NULL) {
 		if (type == MEDIA_SND_CAP) {
+#if QT_VERSION < 0x060000
 			QStringList devAddr = QString(devname).split(QRegExp("[:,]"));
 			return devAddr.value(1).toInt();
+#else
+			QRegExp rx("[:,]");
+			rx.indexIn(devname);
+			return rx.cap(1).toInt();
+#endif
 		}
 	}
 	return -1;
